package com.sap.research.dao;

import java.util.ArrayList;
import java.util.List;

import com.google.appengine.api.datastore.DatastoreService;
import com.google.appengine.api.datastore.DatastoreServiceFactory;
import com.google.appengine.api.datastore.Entity;
import com.google.appengine.api.datastore.EntityNotFoundException;
import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.PreparedQuery;
import com.google.appengine.api.datastore.Query;
import com.sap.research.model.Publication;

public class PublicationNoSqlDao implements PublicationDao {

	private DatastoreService dsService = 
			DatastoreServiceFactory.getDatastoreService();
	
	/* (non-Javadoc)
	 * @see com.sap.research.dao.PublicationDao#persist(com.sap.research.model.Publication, java.lang.String)
	 */
	@Override
	public Key persist(Publication publication, String ... keywords){
		Entity entity = null;
		if(publication.getKey() == null){
			entity = new Entity("Publication");
		}
		else
			entity = new Entity(publication.getKey());
		
		entity.setProperty("title", publication.getTitle());
		entity.setProperty("author", publication.getAuthor());
		Key key = dsService.put(entity);
		publication.setKey(key);
		
		for (String string : keywords) {
			Entity keywordEntity = new Entity("Keyword", key);
			keywordEntity.setProperty("value", string);
			dsService.put(keywordEntity);
		}
		return key;
	}
	
	/* (non-Javadoc)
	 * @see com.sap.research.dao.PublicationDao#load(com.google.appengine.api.datastore.Key)
	 */
	@Override
	public Publication load(Key key) throws EntityNotFoundException{
		Entity entity = dsService.get(key);
		return toPublication(entity);
	}

	private Publication toPublication(Entity entity) {
		return new Publication((String) entity.getProperty("title"), (String) entity.getProperty("author"));
	}
	
	/* (non-Javadoc)
	 * @see com.sap.research.dao.PublicationDao#getAllPublications()
	 */
	@Override
	public List<Publication> getAllPublications(){
		List<Publication> results = new ArrayList<Publication>();
		Query query = new Query("Publication");
		PreparedQuery preparedQuery = dsService.prepare(query);
		for(Entity entity : preparedQuery.asIterable()){
			Publication publication = toPublication(entity);
			results.add(publication);
		}
		return results;
	}
	
	/* (non-Javadoc)
	 * @see com.sap.research.dao.PublicationDao#getAllKeywords(com.google.appengine.api.datastore.Key)
	 */
	@Override
	public List<String> getAllKeywords(Key key){
		List<String> results = new ArrayList<String>();
		Query query = new Query("Keyword");
		query.setAncestor(key);
		PreparedQuery preparedQuery = dsService.prepare(query);
		for(Entity entity : preparedQuery.asIterable()){
			results.add((String) entity.getProperty("value"));
		}
		return results;
	}

}
